/*
 * Copyright (c) 2014 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *					Anthony Mallet on Tue Feb 18 2014
 */
#include "acoptitrack.h"

#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "optitrack_c_types.h"
#include "codels.h"


/* --- Attribute set_logfile -------------------------------------------- */

/** Validation codel set_logfile of attribute set_logfile.
 *
 * Returns genom_ok.
 * Throws optitrack_e_access.
 */
genom_event
set_logfile(const char logfile[256], optitrack_log_s **log,
            genom_context self)
{
  optitrack_e_access_detail d;
  char path[PATH_MAX];
  genom_event e;
  struct tm tm;
  time_t t;
  size_t s;
  FILE *f;

  t = time(NULL);
  localtime_r(&t, &tm);

  s = strftime(path, sizeof(path), logfile, &tm);
  if (s == 0 || s >= sizeof(path)-1) {
    snprintf(d.what, sizeof(d.what), "file name too long");
    return optitrack_e_access(&d, self);
  }

  f = fopen(path, "w");
  if (!f) {
    snprintf(d.what, sizeof(d.what), "%s", strerror(errno));
    return optitrack_e_access(&d, self);
  }
  fprintf(f, "# name time (sec nsec)   x y z   qw qx qy qz  err\n");

  e = unset_logfile(log, self);
  if (e != genom_ok) return e;

  *log = malloc(sizeof(**log));
  if (!*log) return optitrack_e_nomem(self);
  (*log)->f = f;

  return genom_ok;
}


/* --- Function refresh ------------------------------------------------- */

/** Codel refresh of function refresh.
 *
 * Returns genom_ok.
 * Throws optitrack_e_connection.
 */
genom_event
refresh(optitrack_server_s **server, genom_context self)
{
  if (!server || !*server) return optitrack_e_connection(self);

  (*server)->descr = NULL; /* refetch descriptions */
  return genom_ok;
}


/* --- Function unset_logfile ------------------------------------------- */

/** Codel unset_logfile of function unset_logfile.
 *
 * Returns genom_ok.
 * Throws .
 */
genom_event
unset_logfile(optitrack_log_s **log, genom_context self)
{
  if (!log || !*log) return genom_ok;
  if ((*log)->f) fclose((*log)->f);
  free(*log);
  *log = NULL;
  return genom_ok;
}
